/*-------------------------------------------------------------------------
 *
 * sem.h
 *	  System V Semaphore Emulation
 *
 * Copyright (c) 1999, repas AEG Automation GmbH
 *
 *
 * IDENTIFICATION
 *	  $Header: /cvsroot/pgsql/src/backend/port/qnx4/sem.h,v 1.7 2001/11/08 20:37:52 momjian Exp $
 *
 *-------------------------------------------------------------------------
 */

#ifndef _SYS_SEM_H
#define _SYS_SEM_H

#include <nks/synch.h>
#include <sys/types.h>

typedef int key_t;

/*
 *	Semctl Command Definitions.
 */

#define GETNCNT 3						/* get semncnt		*/
#define GETPID	4						/* get sempid		*/
#define GETVAL	5						/* get semval		*/
#define GETALL	6						/* get all semval's */
#define GETZCNT 7						/* get semzcnt		*/
#define SETVAL	8						/* set semval		*/
#define SETALL	9						/* set all semval's */


#define IPC_CREAT	 0001000			/* create key if key does not exist.	*/
#define IPC_EXCL	 0002000			/* fail if key exists.					*/
#define IPC_NOWAIT	 0004000			/* return error on wait.				*/
#define SEM_UNDO	 0010000

#define IPC_PRIVATE		 254			/* create private memory segment		*/

/* control commands for 'semctl', and 'shmctl'.	*/
#define IPC_RMID		   0			/* remove identifier.					*/
#define IPC_SET			   1			/* set `ipc_perm' options.				*/
#define IPC_STAT		   2			/* get `ipc_perm' options.				*/
#define IPC_INFO		   3			/* see ipcs.							*/

/*
 *	There is one semaphore structure for each semaphore in the system.
 */

struct sem
{
	ushort_t		semval;				/* semaphore text map address			*/
	ushort_t		semadj;
	pid_t			sempid;				/* pid of last operation				*/
	ushort_t		semncnt;			/* # awaiting semval > cval 			*/
	ushort_t		semzcnt;			/* # awaiting semval = 0				*/
	NXSema_t		*nxsem_ptr;			/* NX Semaphore pointer					*/
};

struct semid_ds
{
	struct sem		*sem_base;			/* pointer to first semaphore in set	*/
	unsigned short	sem_nsems;			/* number of semaphores in set			*/
	uint64_t		sem_otime;			/* last semop time						*/
	uint64_t		sem_ctime;			/* last change time						*/
	NXSema_t		*base_nxsem_ptr;	/* NX Semaphore pointer					*/
	int				sem_internal_key;	/* 'real' semaphore key					*/
};

struct sembuf
{
	ushort_t		sem_num;			/* semaphore #							*/
	short			sem_op;				/* semaphore operation					*/
	short			sem_flg;			/* operation flags						*/
};

union semun
{
	int				val;
	struct			semid_ds *buf;
	unsigned short	*array;
};

union semun arg;

#ifdef __cplusplus
extern		"C"
{
#endif

int	semctl(int semid, int semnum, int cmd, /* ... */ union semun arg);
int	semget(key_t key, int nsems, int semflg);
int	semop(int semid, struct sembuf * sops, size_t nsops);

#ifdef __cplusplus
}
#endif

#endif   /* _SYS_SEM_H */
